%SQUARE_HOLE_DOMAIN square domain with central square hole structured grid generator 
%
% Grid defining data is saved to the file: squareHole_grid.mat
%
%   TIFISS scriptfile: AB; 07 June 2017.
% Copyright (c) 2017 A. Bespalov, L. Rocchi
%
%
% NOTE
% -------------------------------------------------------------------------
% The domain is created in [0,1]^2 but it is mapped onto [-1,1]^2 if the 
% K-L expansion (rd_type==1) is chosen

  
% Vertices map
  xy = [ 0.000    0.000; ...
         0.250    0.000; ...
         0.500    0.000; ...
         0.750    0.000; ...
         1.000    0.000; ...
         0.000    0.250; ...
         0.250    0.250; ...
         0.500    0.250; ...
         0.750    0.250; ...
         1.000    0.250; ...
         0.375    0.375; ...  
         0.625    0.375; ...
         0.000    0.500; ...  
         0.250    0.500; ...
         0.750    0.500; ...
         1.000    0.500; ...  
         0.375    0.625; ...
         0.625    0.625; ...
         0.000    0.750; ...
         0.250    0.750; ...
         0.500    0.750; ...
         0.750    0.750; ...
         1.000    0.750; ...
         0.000    1.000; ...
         0.250    1.000; ...
         0.500    1.000; ...
         0.750    1.000; ...
         1.000    1.000];
     
% If K-L example is chosen, the domain has to be symmetric with respect 
% to the origin (in [-1,1]^2)     
  if rd_type == 1   
      % Mapping the coordinates from [0,1]^2 onto [-1,1]^2 via the  
      % transform T : [0,1] -> [-1,1], T(\xi) = 2*\xi - 1
      xy = [2*xy(:,1) - 1, 2*xy(:,2) - 1];
  end
        
% Elements map
  evt = [ 7     6     1; ...
          1     2     7; ...
          7     2     3; ...
          3     8     7; ...
          8    11     7; ...
          7    11    14; ...
         17    14    11; ...
          7    14    13; ...
         13     6     7; ...
         
          9     8     3; ...
          3     4     9; ...
          9     4     5; ...
          5    10     9; ...
          9    10    16; ...
         16    15     9; ...       
         15    12     9; ...
         11     8    12; ...
          9    12     8; ...
          
         20    19    13; ...
         13    14    20; ...
         14    17    20; ...
         20    17    21; ...
         18    21    17; ...
         20    21    26; ...       
         26    25    20; ...
         20    25    24; ...
         24    19    20; ...
         
         21    18    22; ...
         22    18    15; ...
         12    15    18; ...
         22    15    16; ...
         16    23    22; ...
         22    23    28; ...
         28    27    22; ...
         22    27    26; ...
         26    21    22];     


% Get boundary nodes and edges
  [eboundt,bound] = find_boundary_elements(evt);
  
% % Total nodes and interior nodes
%   totalnodes = 1:size(xy,1);
%   intnodes = totalnodes(~ismember(totalnodes,bound))
%   plot_mesh(evt,xy,'',1,1,intnodes,bound,eboundt);
%   pause

% Save data
  gohome;
  cd datafiles;
  save squareHole_grid.mat xy evt bound eboundt; 
